@@ -12,13 +12,9 @@
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
-#ifndef _WIN64
-#   define _USE_32BIT_TIME_T
-#endif
 #include "tclWinInt.h"
 #include "tclFileSystem.h"
 #include <winioctl.h>
-#include <sys/stat.h>
 #include <shlobj.h>
 #include <lmaccess.h>		/* For TclpGetUserHome(). */
 
@@ -193,7 +189,7 @@ static unsigned short	NativeStatMode(DWORD attr, int checkLinks,
 			    int isExec);
 static int		NativeIsExec(const TCHAR *path);
 static int		NativeReadReparse(const TCHAR *LinkDirectory,
-			    REPARSE_DATA_BUFFER *buffer);
+			    REPARSE_DATA_BUFFER *buffer, DWORD desiredAccess);
 static int		NativeWriteReparse(const TCHAR *LinkDirectory,
 			    REPARSE_DATA_BUFFER *buffer);
 static int		NativeMatchType(int isDrive, DWORD attr,
@@ -484,7 +480,7 @@ TclWinSymLinkCopyDirectory(
     DUMMY_REPARSE_BUFFER dummy;
     REPARSE_DATA_BUFFER *reparseBuffer = (REPARSE_DATA_BUFFER *) &dummy;
 
-    if (NativeReadReparse(linkOrigPath, reparseBuffer)) {
+    if (NativeReadReparse(linkOrigPath, reparseBuffer, GENERIC_READ)) {
 	return -1;
     }
     return NativeWriteReparse(linkCopyPath, reparseBuffer);
@@ -583,7 +579,7 @@ WinReadLinkDirectory(
     if (!(attr & FILE_ATTRIBUTE_REPARSE_POINT)) {
 	goto invalidError;
     }
-    if (NativeReadReparse(linkDirPath, reparseBuffer)) {
+    if (NativeReadReparse(linkDirPath, reparseBuffer, 0)) {
 	return NULL;
     }
 
@@ -702,12 +698,13 @@ WinReadLinkDirectory(
 static int
 NativeReadReparse(
     const TCHAR *linkDirPath,	/* The junction to read */
-    REPARSE_DATA_BUFFER *buffer)/* Pointer to buffer. Cannot be NULL */
+    REPARSE_DATA_BUFFER *buffer,/* Pointer to buffer. Cannot be NULL */
+    DWORD desiredAccess)
 {
     HANDLE hFile;
     DWORD returnedLength;
 
-    hFile = (*tclWinProcs->createFileProc)(linkDirPath, GENERIC_READ, 0,
+    hFile = (*tclWinProcs->createFileProc)(linkDirPath, desiredAccess, 0,
 	    NULL, OPEN_EXISTING,
 	    FILE_FLAG_OPEN_REPARSE_POINT|FILE_FLAG_BACKUP_SEMANTICS, NULL);
 
@@ -1546,21 +1543,35 @@ NativeAccess(
 
     if (attr == 0xffffffff) {
 	/*
-	 * File doesn't exist.
+	 * File might not exist.
 	 */
 
-	TclWinConvertError(GetLastError());
-	return -1;
+	DWORD lasterror = GetLastError();
+	if (lasterror != ERROR_SHARING_VIOLATION) {
+	    TclWinConvertError(lasterror);
+	    return -1;
+	}
+    }
+
+    if (mode == F_OK) {
+	/*
+	 * File exists, nothing else to check.
+	 */
+
+	return 0;
     }
 
     if ((mode & W_OK)
-	    && (tclWinProcs->getFileSecurityProc == NULL)
-	    && (attr & FILE_ATTRIBUTE_READONLY)) {
+	&& (attr & FILE_ATTRIBUTE_READONLY)
+	&& !(attr & FILE_ATTRIBUTE_DIRECTORY)) {
 	/*
-	 * We don't have the advanced 'getFileSecurityProc', and our
-	 * attributes say the file is not writable. If we do have
-	 * 'getFileSecurityProc', we'll do a more robust XP-related check
-	 * below.
+	 * The attributes say the file is not writable.	 If the file is a
+	 * regular file (i.e., not a directory), then the file is not
+	 * writable, full stop.	 For directories, the read-only bit is
+	 * (mostly) ignored by Windows, so we can't ascertain anything about
+	 * directory access from the attrib data.  However, if we have the
+	 * advanced 'getFileSecurityProc', then more robust ACL checks
+	 * will be done below.
 	 */
 
 	Tcl_SetErrno(EACCES);
@@ -1584,15 +1595,14 @@ NativeAccess(
      * we have a more complex permissions structure so we try to check that.
      * The code below is remarkably complex for such a simple thing as finding
      * what permissions the OS has set for a file.
-     *
-     * If we are simply checking for file existence, then we don't need all
-     * these complications (which are really quite slow: with this code 'file
-     * readable' is 5-6 times slower than 'file exists').
      */
 
-    if ((mode != F_OK) && (tclWinProcs->getFileSecurityProc != NULL)) {
+    if (tclWinProcs->getFileSecurityProc != NULL) {
 	SECURITY_DESCRIPTOR *sdPtr = NULL;
 	unsigned long size;
+	PSID pSid = 0;
+	BOOL SidDefaulted;
+	SID_IDENTIFIER_AUTHORITY samba_unmapped = {{0, 0, 0, 0, 0, 22}};
 	GENERIC_MAPPING genMap;
 	HANDLE hToken = NULL;
 	DWORD desiredAccess = 0, grantedAccess = 0;
@@ -1608,7 +1618,8 @@ NativeAccess(
 	size = 0;
 	(*tclWinProcs->getFileSecurityProc)(nativePath,
 		OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION
-		| DACL_SECURITY_INFORMATION, 0, 0, &size);
+		| DACL_SECURITY_INFORMATION | LABEL_SECURITY_INFORMATION,
+		0, 0, &size);
 
 	/*
 	 * Should have failed with ERROR_INSUFFICIENT_BUFFER
@@ -1641,7 +1652,8 @@ NativeAccess(
 
 	if (!(*tclWinProcs->getFileSecurityProc)(nativePath,
 		OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION
-		| DACL_SECURITY_INFORMATION, sdPtr, size, &size)) {
+		| DACL_SECURITY_INFORMATION | LABEL_SECURITY_INFORMATION,
+		sdPtr, size, &size)) {
 	    /*
 	     * Error getting owner SD
 	     */
@@ -1649,6 +1661,26 @@ NativeAccess(
 	    goto accessError;
 	}
 
+	/*
+	 * As of Samba 3.0.23 (10-Jul-2006), unmapped users and groups are
+	 * assigned to SID domains S-1-22-1 and S-1-22-2, where "22" is the
+	 * top-level authority.	 If the file owner and group is unmapped then
+	 * the ACL access check below will only test against world access,
+	 * which is likely to be more restrictive than the actual access
+	 * restrictions.  Since the ACL tests are more likely wrong than
+	 * right, skip them.  Moreover, the unix owner access permissions are
+	 * usually mapped to the Windows attributes, so if the user is the
+	 * file owner then the attrib checks above are correct (as far as they
+	 * go).
+	 */
+
+	if(!GetSecurityDescriptorOwner(sdPtr,&pSid,&SidDefaulted) ||
+	   memcmp(GetSidIdentifierAuthority(pSid),&samba_unmapped,
+		  sizeof(SID_IDENTIFIER_AUTHORITY))==0) {
+	    HeapFree(GetProcessHeap(), 0, sdPtr);
+	    return 0; /* Attrib tests say access allowed. */
+	}
+
 	/*
 	 * Perform security impersonation of the user and open the resulting
 	 * thread token.
@@ -1726,17 +1758,6 @@ NativeAccess(
 	    return -1;
 	}
 
-	/*
-	 * For directories the above checks are ok. For files, though, we must
-	 * still check the 'attr' value.
-	 */
-
-	if ((mode & W_OK)
-		&& !(attr & FILE_ATTRIBUTE_DIRECTORY)
-		&& (attr & FILE_ATTRIBUTE_READONLY)) {
-	    Tcl_SetErrno(EACCES);
-	    return -1;
-	}
     }
     return 0;
 }
@@ -1832,27 +1853,10 @@ TclpObjChdir(
 {
     int result;
     const TCHAR *nativePath;
-#ifdef __CYGWIN__
-    extern int cygwin_conv_to_posix_path(const char *, char *);
-    char posixPath[MAX_PATH+1];
-    const char *path;
-    Tcl_DString ds;
-#endif /* __CYGWIN__ */
 
     nativePath = (const TCHAR *) Tcl_FSGetNativePath(pathPtr);
 
-#ifdef __CYGWIN__
-    /*
-     * Cygwin chdir only groks POSIX path.
-     */
-
-    path = Tcl_WinTCharToUtf(nativePath, -1, &ds);
-    cygwin_conv_to_posix_path(path, posixPath);
-    result = (chdir(posixPath) == 0 ? 1 : 0);
-    Tcl_DStringFree(&ds);
-#else /* __CYGWIN__ */
     result = (*tclWinProcs->setCurrentDirectoryProc)(nativePath);
-#endif /* __CYGWIN__ */
 
     if (result == 0) {
 	TclWinConvertError(GetLastError());
@@ -1861,51 +1865,6 @@ TclpObjChdir(
     return 0;
 }
 
-#ifdef __CYGWIN__
-/*
- *---------------------------------------------------------------------------
- *
- * TclpReadlink --
- *
- *	This function replaces the library version of readlink().
- *
- * Results:
- *	The result is a pointer to a string specifying the contents of the
- *	symbolic link given by 'path', or NULL if the symbolic link could not
- *	be read. Storage for the result string is allocated in bufferPtr; the
- *	caller must call Tcl_DStringFree() when the result is no longer
- *	needed.
- *
- * Side effects:
- *	See readlink() documentation.
- *
- *---------------------------------------------------------------------------
- */
-
-char *
-TclpReadlink(
-    const char *path,		/* Path of file to readlink (UTF-8). */
-    Tcl_DString *linkPtr)	/* Uninitialized or free DString filled with
-				 * contents of link (UTF-8). */
-{
-    char link[MAXPATHLEN];
-    int length;
-    char *native;
-    Tcl_DString ds;
-
-    native = Tcl_UtfToExternalDString(NULL, path, -1, &ds);
-    length = readlink(native, link, sizeof(link));	/* INTL: Native. */
-    Tcl_DStringFree(&ds);
-
-    if (length < 0) {
-	return NULL;
-    }
-
-    Tcl_ExternalToUtfDString(NULL, link, length, linkPtr);
-    return Tcl_DStringValue(linkPtr);
-}
-#endif /* __CYGWIN__ */
-
 /*
  *----------------------------------------------------------------------
  *
@@ -2090,8 +2049,21 @@ NativeStat(
 
 	if ((*tclWinProcs->getFileAttributesExProc)(nativePath,
 		GetFileExInfoStandard, &data) != TRUE) {
-	    Tcl_SetErrno(ENOENT);
-	    return -1;
+	    HANDLE hFind;
+	    WIN32_FIND_DATAT ffd;
+	    DWORD lasterror = GetLastError();
+
+	    if (lasterror != ERROR_SHARING_VIOLATION) {
+		TclWinConvertError(lasterror);
+		return -1;
+		}
+	    hFind = (*tclWinProcs->findFirstFileProc)(nativePath, &ffd);
+	    if (hFind == INVALID_HANDLE_VALUE) {
+		TclWinConvertError(GetLastError());
+		return -1;
+	    }
+	    memcpy(&data, &ffd, sizeof(data));
+	    FindClose(hFind);
 	}
 
 	attr = data.dwFileAttributes;
@@ -2613,6 +2585,12 @@ TclpObjNormalizePath(
 				}
 				Tcl_DStringAppend(&dsNorm, nativePath, len);
 				lastValidPathEnd = currentPathEndPosition;
+			    } else if (nextCheckpoint == 0) {
+				/* Path starts with a drive designation
+				 * that's not actually on the system.
+				 * We still must normalize up past the
+				 * first separator.  [Bug 3603434] */
+				currentPathEndPosition++;
 			    }
 			}
 			Tcl_DStringFree(&ds);
@@ -2755,6 +2733,12 @@ TclpObjNormalizePath(
 			    Tcl_DStringAppend(&dsNorm, nativePath,
 				    (int)(sizeof(WCHAR) * len));
 			    lastValidPathEnd = currentPathEndPosition;
+			} else if (nextCheckpoint == 0) {
+			    /* Path starts with a drive designation
+			     * that's not actually on the system.
+			     * We still must normalize up past the
+			     * first separator.  [Bug 3603434] */
+			    currentPathEndPosition++;
 			}
 		    }
 		    Tcl_DStringFree(&ds);
